@@ -17,6 +17,7 @@
 #include "nvspdif.h"
 
 extern struct Nvaudio_card *devs;
+extern unsigned int spdif_status;
 
 
 /* in this loop, dmabuf.count signifies the amount of data that is
@@ -422,8 +423,14 @@ static int Nvaudio_mmapwrite(struct vm_area_struct *vma,struct Nvaudio_state *st
         if (size > (PAGE_SIZE << dmabuf->buforder))
            return ret;
         ret = -EAGAIN;
+
+#if defined(REMAP_NEW)
+        if (remap_page_range(vma, vma->vm_start, virt_to_phys(dmabuf->rawbuf),
+                 size, vma->vm_page_prot))
+#else
         if (remap_page_range(vma->vm_start, virt_to_phys(dmabuf->rawbuf),
                  size, vma->vm_page_prot))
+#endif
            return ret;
     }
     state->mapped  = 1;
@@ -461,8 +468,14 @@ static int Nvaudio_mmap(struct file *file, struct vm_area_struct *vma)
         if (size > (PAGE_SIZE << recdmabuf->buforder))
           goto out;
         ret = -EAGAIN;
+
+#if defined(REMAP_NEW)
+        if (remap_page_range(vma, vma->vm_start, virt_to_phys(recdmabuf->rawbuf),
+                 size, vma->vm_page_prot))
+#else
         if (remap_page_range(vma->vm_start, virt_to_phys(recdmabuf->rawbuf),
                  size, vma->vm_page_prot))
+#endif
            goto out;
         recstate->mapped = 1;
         recstate->trigger = 0;
@@ -475,6 +488,8 @@ static int Nvaudio_mmap(struct file *file, struct vm_area_struct *vma)
           if( (ret =  Nvaudio_mmapwrite(vma,card->spout,2)) != 0)
               goto out;
        }
+       /* Disable the Spdif right now */
+       dacstate->spdif_enable = 0;
 #endif
          ret = Nvaudio_mmapwrite(vma,card->wavout,0);
     }
@@ -853,19 +868,25 @@ static int Nvaudio_ioctl(struct inode *inode, struct file *file, unsigned int cm
 
             switch ( val ) {
                 case 2: /* 2 channels is always supported */
+                    /*Set the channels getting used; */
+                    card->numchannels = val;
+
                     outl(i_glob_cnt & 0xcfffff,
                          card->iobase + GLOB_CNT);
                     /* Do we need to change mixer settings????  */
                     break;
                 case 4: /* Supported on some chipsets, better check first */
                     if ( card->channels >= 4 ) {
+
+                        /*Set the channels getting used; */
+                        card->numchannels = val;
+
                         outl((i_glob_cnt & 0xcfffff) | 0x100000,
                               card->iobase + GLOB_CNT);
+
                         /* Read the Register and set as well*/
-                        Nvaudio_ac97_set(codec,AC97_SURROUND_MASTER,
-                                Nvaudio_ac97_get(codec,AC97_MASTER_VOL_STEREO));
-                        Nvaudio_ac97_set(codec,0x64,
-                                    Nvaudio_ac97_get(codec,AC97_PCMOUT_VOL));
+                        Nvaudio_MultiChannel_volume(card,val);
+                        
 #ifdef NV_DEBUG_NORMAL
                         printk("pcm 0x%x, master 0x%x, sur 0x%x, reg64 0x%x  \n",
                                 Nvaudio_ac97_get(codec,AC97_PCMOUT_VOL),
@@ -880,18 +901,15 @@ static int Nvaudio_ioctl(struct inode *inode, struct file *file, unsigned int cm
                     break;
                 case 6: /* Supported on some chipsets, better check first */
                     if ( card->channels >= 6 ) {
+
+                        /*Set the channels getting used; */
+                        card->numchannels = val;
+
                         outl((i_glob_cnt & 0xcfffff) | 0x200000,
                               card->iobase + GLOB_CNT);
                         /* Read the Register and set as well*/
-                             Nvaudio_ac97_set(codec,AC97_SURROUND_MASTER,
-                                Nvaudio_ac97_get(codec,AC97_MASTER_VOL_STEREO));
-                             Nvaudio_ac97_set(codec,0x64,
-                                    Nvaudio_ac97_get(codec,AC97_PCMOUT_VOL));
-
-                             Nvaudio_ac97_set(codec,AC97_CENTER_LFE_MASTER,
-                                Nvaudio_ac97_get(codec,AC97_MASTER_VOL_STEREO));
-                             Nvaudio_ac97_set(codec,0x66,
-                                    Nvaudio_ac97_get(codec,AC97_PCMOUT_VOL));
+                        Nvaudio_MultiChannel_volume(card,val);
+                        
 #ifdef NV_DEBUG_NORMAL
                              printk(" channel 6 pcm 0x%x, master 0x%x, sur 0x%x , center 0x%x, reg64 0x%x , reg66 0x%x \n",
                                 Nvaudio_ac97_get(codec,AC97_PCMOUT_VOL),
@@ -1482,6 +1500,11 @@ static int Nvaudio_open(struct inode *inode, struct file *file)
                     if((file->f_mode & FMODE_READ) && (card->wavin)) {
                         kfree(card->wavin);
                         card->wavin = NULL;
+#ifdef ENABLE_SPDIF
+                        if(card->spout) {
+                            card->wavout->spdif_enable = 1;
+                        }
+#endif
                     }
                     return -EBUSY;
                 }
@@ -1516,7 +1539,7 @@ static int Nvaudio_open(struct inode *inode, struct file *file)
                 outl(i & 0xffcfffff, card->iobase + GLOB_CNT);
 
                 /* set the channel number to 2 */
-                card->numchannels = 2;
+                card->numchannels = CHANNEL_STEREO;
 
                 /* set default sample format. According to OSS Programmer's Guide  /dev/dsp
                 should be default to unsigned 8-bits, mono, with sample rate 8kHz and
@@ -1532,8 +1555,11 @@ static int Nvaudio_open(struct inode *inode, struct file *file)
                 state->spdif_enable = 0;
 
 #ifdef ENABLE_SPDIF
-                if( 0 == Nvaudio_spdifopen(card))
-                   state->spdif_enable = 1;
+                if(spdif_status) {
+                    if( 0 == Nvaudio_spdifopen(card)) {
+                       state->spdif_enable = 1;
+                    }
+                }
 #endif
             }
 
@@ -1549,8 +1575,9 @@ static int Nvaudio_open(struct inode *inode, struct file *file)
 
 static int Nvaudio_release(struct inode *inode, struct file *file)
 {
-    struct Nvaudio_state *state = NULL;
-    struct dmabuf *dmabuf       = NULL;
+    struct Nvaudio_state *state    = NULL;
+    struct Nvaudio_state *dacstate = NULL;
+    struct dmabuf *dmabuf          = NULL;
     unsigned long flags;
     struct Nvaudio_card *card   = (struct Nvaudio_card *)file->private_data;
 
@@ -1572,18 +1599,19 @@ static int Nvaudio_release(struct inode *inode, struct file *file)
     }
 
     if(file->f_mode & FMODE_WRITE) {
-        state  = card->wavout;
-        dmabuf = &state->dmabuffer;
-        if(state->trigger & PCM_ENABLE_OUTPUT) {
-            drain_dac(state, 0);
+        dacstate  = card->wavout;
+        dmabuf = &dacstate->dmabuffer;
+        if(dacstate->trigger & PCM_ENABLE_OUTPUT) {
+            drain_dac(dacstate, 0);
         }
-        spin_lock_irqsave(&state->lock, flags);
-        dealloc_dmabuf(state);
-        spin_unlock_irqrestore(&state->lock, flags);
-        kfree(state);
-        card->wavout = NULL;
+        spin_lock_irqsave(&dacstate->lock, flags);
+        dealloc_dmabuf(dacstate);
+        spin_unlock_irqrestore(&dacstate->lock, flags);
+        
 #ifdef ENABLE_SPDIF
-    if(state->spdif_enable) {
+        if(card->spout)
+            dacstate->spdif_enable = 1;
+        if(dacstate->spdif_enable) {
            state  = card->spout;
            dmabuf = &state->dmabuffer;
            if(state->trigger & PCM_ENABLE_OUTPUT) {
@@ -1594,8 +1622,10 @@ static int Nvaudio_release(struct inode *inode, struct file *file)
            spin_unlock_irqrestore(&state->lock, flags);
            kfree(state);
            card->spout = NULL;
-    }
+       }
 #endif
+        kfree(dacstate);
+        card->wavout = NULL;
     }
     unlock_kernel();
     return 0;
