@@ -75,9 +75,12 @@ struct Nvaudio_card *devs = NULL;
 
 int strict_clocking = 0;
 unsigned int clocking = 0;
+unsigned int spdif_status = 1;
 
-static int mastervol = 0;
-static int pcmvol    = 0;
+static int mastervolleft  = 0;
+static int pcmvolleft     = 0;
+static int mastervolright = 0;
+static int pcmvolright    = 0;
 
 /* OSS /dev/mixer file operation methods */
 int Nvaudio_open_mixdev(struct inode *inode, struct file *file)
@@ -137,27 +140,51 @@ static int Nvaudio_private_ioctl(struct Nvaudio_card* card, unsigned int cmd, un
 
             case NV_AC97CODEC_WRITE:
                outw(regValue,card->ac97base + regIndex);
+#ifdef NV_DEBUG_NORMAL
                printk("NV_CODEC_WRITE with val %x\n",regValue);
+#endif
                break;
 
             case NV_AC97CODEC_READ:
                nvctl->val[1] = inw(card->ac97base + regIndex);
+#ifdef NV_DEBUG_NORMAL
                printk("NV_CODEC_READ with val %lx\n",nvctl->val[1]);
+#endif
                if( copy_to_user((void *)arg, nvctl, sizeof(struct nv_ioctl)))
                    ret = -EINVAL;
                break;
 
             case NV_AC97CON_WRITE:
+#ifdef NV_DEBUG_NORMAL
                printk("NV_CON_WRITE with val %x\n",regValue);
+#endif
                outw(regValue,card->iobase + regIndex);
                break;
 
             case NV_AC97CON_READ:
                nvctl->val[1] = inw(card->iobase + regIndex);
+#ifdef NV_DEBUG_NORMAL
                printk("NV_CON_READ with val %lx\n",nvctl->val[1]);
+#endif
                if( copy_to_user((void *)arg, nvctl, sizeof(struct nv_ioctl)))
                   ret = -EINVAL;
                break;
+            case NV_SET_SPKRSELECT:
+                card->spkrselect = nvctl->val[0];
+                /* set the Global cnt regsiter as well 
+                Nvaudio_SetSpkrRegister(card);
+                */
+                break;
+
+            case NV_GET_SPKRSELECT:
+                nvctl->val[0] = card->spkrselect;
+                break;
+
+            case NV_SET_ANALOGOUT:
+            case NV_GET_ANALOGOUT:
+            case NV_SET_DIGITALOUT:
+            case NV_GET_DIGITALOUT:
+                break;
 
             default:
                ret = -EINVAL;
@@ -189,13 +216,17 @@ int Nvaudio_ioctl_mixdev(struct inode *inode, struct file *file, unsigned int cm
         get_user(val,(int *)arg);
         switch(i) {
             case SOUND_MIXER_VOLUME: /* Master volume */
-                mastervol = (val >> 8) & 0xff;
-                if(mastervol > 100) mastervol = 100;
+                mastervolright = (val >> 8) & 0xff;
+                mastervolleft  = val & 0xff;
+                if(mastervolright > 100) mastervolright = 100;
+                if(mastervolleft  > 100) mastervolleft  = 100;
                 break;
 
             case SOUND_MIXER_PCM: /* Pcm Vol */
-                pcmvol = (val >> 8) & 0xff;
-                if(pcmvol > 100) pcmvol = 100;
+                pcmvolright = (val >> 8) & 0xff;
+                if(pcmvolright > 100) pcmvolright = 100;
+                pcmvolleft  = val  & 0xff;
+                if(pcmvolleft > 100) pcmvolleft = 100;
                 break;
         }
     }
@@ -212,24 +243,8 @@ int Nvaudio_ioctl_mixdev(struct inode *inode, struct file *file, unsigned int cm
     */
     switch(i) {
         case SOUND_MIXER_VOLUME: /* Master volume */
-            if ( card->channels == 6 ) {
-                Nvaudio_ac97_set(codec,AC97_CENTER_LFE_MASTER,
-                    Nvaudio_ac97_get(codec,AC97_MASTER_VOL_STEREO));
-            }
-            if ( card->channels >= 4 ) {
-                Nvaudio_ac97_set(codec,AC97_SURROUND_MASTER,
-                    Nvaudio_ac97_get(codec,AC97_MASTER_VOL_STEREO));
-            }
-            break;
-        case SOUND_MIXER_PCM: /* Pcm Vol */
-            if ( card->channels == 6 ) {
-                Nvaudio_ac97_set(codec,0x66,
-                    Nvaudio_ac97_get(codec,AC97_PCMOUT_VOL));
-            }
-            if ( card->channels >= 4 ) {
-                Nvaudio_ac97_set(codec,0x64,
-                    Nvaudio_ac97_get(codec,AC97_PCMOUT_VOL));
-            }
+        case SOUND_MIXER_PCM:    /* Pcm Vol */
+            Nvaudio_MultiChannel_volume(card, card->channels);
             break;
     }
 
@@ -279,6 +294,11 @@ static void Nvaudio_interrupt(int irq, void *dev_id, struct pt_regs *regs)
         /* clear 'em */
             outl(status & INT_GPI, card->iobase + GLOB_STA);
         }
+
+        if(status & INT_RCOM) {
+        /* clear 'em */
+            outl(status & INT_RCOM, card->iobase + GLOB_STA);
+        }
         status = 0;
         status = inl(card->iobase + GLOB_STA);
     }
@@ -306,6 +326,7 @@ int Nvaudio_ac97_probe_and_powerup(struct Nvaudio_card *card,struct ac97_codec *
 {
     /* Returns 0 on failure */
     int i;
+    u16 id1 = 0, id2 = 0;
     if (ac97_probe_codec(codec) == 0) return 0;
 
     /* power it all up */
@@ -319,6 +340,36 @@ int Nvaudio_ac97_probe_and_powerup(struct Nvaudio_card *card,struct ac97_codec *
         set_current_state(TASK_UNINTERRUPTIBLE);
         schedule_timeout(HZ/20);
     }
+
+    /* Separate Checking for Sigmatel codec. If POWER_CONTROL registers are
+       not getting set . check for the id and try to initialize the
+       Sigmatel separately.
+     */
+    id1 = codec->codec_read(codec, AC97_VENDOR_ID1);
+    id2 = codec->codec_read(codec, AC97_VENDOR_ID2);
+    
+    if((id1 == AC97REG_SIGMATEL_ID1) &&(id2 == AC97REG_SIGMATEL_ID2)){
+        card->codecsec  = SIGMATEL_SECONDARY;
+        card->codecType = SIGMATEL_CODEC;
+    }
+
+    if((id1 == AC97REG_ADI1885_ID1) && (id2 == AC97REG_ADI1885_ID2))    {
+        card->codecType = ADI_CODEC;
+        // set the codec to use the rear surround slots
+        codec->codec_write(codec, AC97_ADI_MULTICHN, 0x7000);
+    }
+
+    if((i <= 0) && (card->codecsec == SIGMATEL_SECONDARY)){
+        i = 10;
+        // set the codec to use enable the clock access register
+        codec->codec_write(codec, AC97_SIGMATEL_CIC1, 0xABBA);
+        // set the codec to use the BIT_CLK
+        codec->codec_write(codec, AC97_SIGMATEL_CIC2, 0x5400);
+        // enable the surround dac
+        codec->codec_write(codec, AC97_EXTENDED_STATUS, 0x0000);
+        // set the codec to use the rear surround slots
+        codec->codec_write(codec, AC97_SIGMATEL_MULTICHN, 0x0001);
+    }
     return i;
 }
 
@@ -380,6 +431,8 @@ int __init Nvaudio_ac97_init(struct Nvaudio_card *card)
     u16 eid = 0;
     u32 reg = 0;
 
+    card->codecType = 0;
+
     if(!Nvaudio_ac97_random_init_stuff(card)) return 0;
 
     /* Number of channels supported */
@@ -401,6 +454,9 @@ int __init Nvaudio_ac97_init(struct Nvaudio_card *card)
     reg = inl(card->iobase + GLOB_CNT);
     outl(reg & 0xffcfffff, card->iobase + GLOB_CNT);
 
+    /* set the channel number to 2 */
+    card->numchannels = CHANNEL_STEREO;
+
     inw(card->ac97base);
 
     for (num_ac97 = 0; num_ac97 < NR_AC97; num_ac97++) {
@@ -432,6 +488,8 @@ int __init Nvaudio_ac97_init(struct Nvaudio_card *card)
         codec->codec_read   = Nvaudio_ac97_get;
         codec->codec_write  = Nvaudio_ac97_set;
 
+        card->codecsec      = NO_SECONDARY;
+
         if(!Nvaudio_ac97_probe_and_powerup(card,codec)) {
             printk("Nvaudio: timed out waiting for codec %d analog ready", num_ac97);
             kfree(codec);
@@ -453,7 +511,8 @@ int __init Nvaudio_ac97_init(struct Nvaudio_card *card)
 
         /* As Spdif support only 48K - restrict the ac97 to play in 48K always*/
         /* added for nvaudio to do spdif support */
-        eid &= ~1;
+        if(spdif_status)
+            eid &= ~1;
         card->ac97_features = eid;
 
 #ifdef NV_DEBUG_NORMAL
@@ -495,43 +554,52 @@ int __init Nvaudio_ac97_init(struct Nvaudio_card *card)
         if((eid & 0xc000) == 0) /* primary codec */
             total_channels += 2;
 
-        if(eid & 0x200) { /* GOOD, AMAP support */
-            if (eid & 0x0080) /* L/R Surround channels */
-                total_channels += 2;
-            if (eid & 0x0140) /* LFE and Center channels */
-                total_channels += 2;
-            printk("Nvaudio: AC'97 codec %d supports AMAP, total channels = %d\n", num_ac97, total_channels);
-        } else if (eid & 0x0400) {  /* this only works on 2.2 compliant codecs */
-            eid &= 0xffcf;
-            if((eid & 0xc000) != 0) {
-                switch ( total_channels ) {
-                    case 2:
-                        /* Set dsa1, dsa0 to 01 */
-                        eid |= 0x0010;
-                        break;
-                    case 4:
-                        /* Set dsa1, dsa0 to 10 */
-                        eid |= 0x0020;
-                        break;
-                    case 6:
-                        /* Set dsa1, dsa0 to 11 */
-                        eid |= 0x0030;
-                        break;
-                }
-                total_channels += 2;
+        if(card->codecsec == SIGMATEL_SECONDARY)    {
+            if((eid & 0x0080) != 0)  { //SDAC set
+                total_channels += 4;
             }
-            Nvaudio_ac97_set(codec, AC97_EXTENDED_ID, eid);
-            eid = Nvaudio_ac97_get(codec, AC97_EXTENDED_ID);
-            printk("Nvaudio: AC'97 codec %d, new EID value = 0x%04x\n", num_ac97, eid);
-            if (eid & 0x0080) /* L/R Surround channels */
-                total_channels += 2;
-            if (eid & 0x0140) /* LFE and Center channels */
-                total_channels += 2;
-            printk("Nvaudio: AC'97 codec %d, DAC map configured, total channels = %d\n", num_ac97, total_channels);
+            printk("Nvaudio: AC'97 codec %d, total channels = %d\n", num_ac97, total_channels);
+        
         } else {
-            printk("Nvaudio: AC'97 codec %d Unable to map surround DAC's (or DAC's not present), total channels = %d\n", num_ac97, total_channels);
-        }
+        
+            if(eid & 0x200) { /* GOOD, AMAP support */
+                if (eid & 0x0080) /* L/R Surround channels */
+                    total_channels += 2;
+                if (eid & 0x0140) /* LFE and Center channels */
+                    total_channels += 2;
+                printk("Nvaudio: AC'97 codec %d supports AMAP, total channels = %d\n", num_ac97, total_channels);
 
+            } else if (eid & 0x0400) {  /* this only works on 2.2 compliant codecs */
+                eid &= 0xffcf;
+                if((eid & 0xc000) != 0) {
+                    switch ( total_channels ) {
+                        case 2:
+                            /* Set dsa1, dsa0 to 01 */
+                            eid |= 0x0010;
+                            break;
+                        case 4:
+                            /* Set dsa1, dsa0 to 10 */
+                            eid |= 0x0020;
+                            break;
+                        case 6:
+                            /* Set dsa1, dsa0 to 11 */
+                            eid |= 0x0030;
+                            break;
+                    }
+                    total_channels += 2;
+                }
+                Nvaudio_ac97_set(codec, AC97_EXTENDED_ID, eid);
+                eid = Nvaudio_ac97_get(codec, AC97_EXTENDED_ID);
+                if (eid & 0x0080) /* L/R Surround channels */
+                    total_channels += 2;
+                if (eid & 0x0140) /* LFE and Center channels */
+                    total_channels += 2;
+                printk("Nvaudio: AC'97 codec %d, DAC map configured, total channels = %d\n", num_ac97, total_channels);
+            } else {
+                if(card->codecType != ADI_CODEC)
+                    printk("Nvaudio: AC'97 codec %d Unable to map surround DAC's (or DAC's not present), total channels = %d\n", num_ac97, total_channels);
+            }
+        }
         if ((codec->dev_mixer = register_sound_mixer(&Nvaudio_mixer_fops, -1)) < 0) {
             printk(KERN_ERR "Nvaudio: couldn't register mixer!\n");
             kfree(codec);
@@ -540,12 +608,25 @@ int __init Nvaudio_ac97_init(struct Nvaudio_card *card)
 
         card->ac97_codec[num_ac97] = codec;
 
-        /* Get the master vol and pcmvol for spdif - need to take separate for multiple codec */
-        mastervol = ( codec->read_mixer(codec,SOUND_MIXER_VOLUME)  >> 8 ) & 0xff;
-        if(mastervol > 100) mastervol = 100;
+        if(card->codecsec == NO_SECONDARY) {
+            /* Get the master vol and pcmvol for spdif - need to take separate for multiple codec */
+            mastervolleft = ( codec->read_mixer(codec,SOUND_MIXER_VOLUME)  >> 8 ) & 0xff;
+            if(mastervolleft > 100) mastervolleft = 100;
+            mastervolright =  codec->read_mixer(codec,SOUND_MIXER_VOLUME) & 0xff;
+            if(mastervolright > 100) mastervolright = 100;
 
-        pcmvol = ( codec->read_mixer(codec,SOUND_MIXER_PCM)  >> 8 ) & 0xff;
-        if(pcmvol > 100) mastervol = 100;
+
+            pcmvolleft = ( codec->read_mixer(codec,SOUND_MIXER_PCM)  >> 8 ) & 0xff;
+            if(pcmvolleft > 100) pcmvolleft = 100;
+            pcmvolright = codec->read_mixer(codec,SOUND_MIXER_PCM) & 0xff;
+            if(pcmvolright > 100) pcmvolright = 100;
+        }
+
+        if(card->codecType == ADI_CODEC) {
+            total_channels += 4;
+            num_ac97       += 1;
+            break;
+        }
     }
 
     /* pick the minimum of channels supported by ICHx or codec(s) */
@@ -718,7 +799,8 @@ int __init Nvaudio_probe(struct pci_dev *pci_dev, const struct pci_device_id *pc
         goto out_pio;
     }
     card->initializing = 0;
-    printk(KERN_INFO"Nvaudio: PROBE is done \n");
+    /* set the Spkr as Stereo by default*/
+    card->spkrselect  = NV_SPKR_STEREO;
     return 0;
 
 out_pio:
@@ -881,12 +963,16 @@ int Nvaudio_pm_resume(struct pci_dev *dev)
                            ((val >> 8)  & 0xff) );
                 /* set the volumes for spdif control */
                 if(i == SOUND_MIXER_VOLUME) { /* Master volume */
-                    mastervol = (val >> 8) & 0xff;
-                    if(mastervol > 100) mastervol = 100;
+                    mastervolright = (val >> 8) & 0xff;
+                    if(mastervolright > 100) mastervolright = 100;
+                    mastervolleft = val & 0xff;
+                    if(mastervolleft > 100) mastervolleft = 100;
                 }
                 if(i == SOUND_MIXER_PCM) { /* Pcm Vol */
-                    pcmvol = (val >> 8) & 0xff;
-                    if(pcmvol > 100) pcmvol = 100;
+                    pcmvolright = (val >> 8) & 0xff;
+                    if(pcmvolright > 100) pcmvolright = 100;
+                    pcmvolleft = val  & 0xff;
+                    if(pcmvolleft > 100) pcmvolleft = 100;
                 }
             }
         }
@@ -931,6 +1017,7 @@ MODULE_DESCRIPTION("Nvidia audio support");
 MODULE_LICENSE("GPL");
 MODULE_PARM(clocking, "i");
 MODULE_PARM(strict_clocking, "i");
+MODULE_PARM(spdif_status, "i");
 
 static struct pci_driver Nvaudio_pci_driver = {
     name:       Nvaudio_MODULE_NAME,
