@@ -14,8 +14,10 @@
 
 extern int strict_clocking;
 extern unsigned int clocking;
-extern int mastervol;
-extern int pcmvol;
+extern int mastervolright;
+extern int mastervolleft;
+extern int pcmvolright;
+extern int pcmvolleft;
 
 static short SPDIFLeftovers[6];
 static u32 SPDIFLeftoverCount = 0;
@@ -325,6 +327,7 @@ int Nvaudio_spdifopen(struct Nvaudio_card  *card)
     dmabuf->ossmaxfrags  = 0;
     dmabuf->subdivision  = 0;
 
+    state->mapped     = 0;
     state->open_mode |= FMODE_WRITE;
     spin_lock_init(&state->lock);
 
@@ -473,8 +476,8 @@ ssize_t Nvaudio_spdifwrite(struct Nvaudio_card *card, const char *buffer, size_t
                         right   = *pSource;
                         pSource += 1;
 
-                        left   = (left * mastervol * pcmvol) / 10000; 
-                        right  = (right * mastervol * pcmvol) / 10000; 
+                        left   = (left * mastervolleft * pcmvolleft) / 10000; 
+                        right  = (right * mastervolright * pcmvolright) / 10000; 
 
                         *pDest = (short) left;
                         pDest  += 1;
@@ -509,8 +512,8 @@ ssize_t Nvaudio_spdifwrite(struct Nvaudio_card *card, const char *buffer, size_t
                             right = -32768;
 
                         /* apply the volume */
-                        left   = ( left * mastervol * pcmvol) / 10000; 
-                        right  = ( right * mastervol * pcmvol) / 10000; 
+                        left   = ( left * mastervolleft * pcmvolleft) / 10000; 
+                        right  = ( right * mastervolright * pcmvolright) / 10000; 
 
                         *pDest = (short)left;
                         pDest += 1;
@@ -625,8 +628,8 @@ ssize_t Nvaudio_spdifwrite(struct Nvaudio_card *card, const char *buffer, size_t
 
 
                         /* apply the volume */
-                        left   = ( left * mastervol * pcmvol) / 10000; 
-                        right  = ( right * mastervol * pcmvol) / 10000; 
+                        left   = ( left * mastervolleft * pcmvolleft) / 10000; 
+                        right  = ( right * mastervolright * pcmvolright) / 10000; 
 
                         *pDest = (short)left;
                         pDest += 1;
@@ -667,8 +670,8 @@ ssize_t Nvaudio_spdifwrite(struct Nvaudio_card *card, const char *buffer, size_t
                             right = -32768;
 
                         /* apply the volume */
-                        left   = ( left * mastervol * pcmvol) / 10000; 
-                        right  = ( right * mastervol * pcmvol) / 10000; 
+                        left   = ( left * mastervolleft * pcmvolleft) / 10000; 
+                        right  = ( right * mastervolright * pcmvolright) / 10000; 
 
                         *pDest = (short)left;
                         pDest += 1;
